@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: readerctapi.h 122 2005-10-22 00:42:09Z aquamaniac $
+    cvs         : $Id: readerctapi.h 378 2007-05-02 08:41:05Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -15,15 +15,10 @@
 #define CHIPCARD_READER_CTAPI_H
 
 #include <gwenhywfar/libloader.h>
-#include "reader_l.h"
+#include <chipcard/server/driver/reader.h>
 
 
-LCD_READER *ReaderCTAPI_new(GWEN_TYPE_UINT32 readerId,
-                           const char *name,
-                           int port,
-                           unsigned int slots,
-                           GWEN_TYPE_UINT32 flags,
-                           int ctn);
+int ReaderCTAPI_Extend(LCD_READER *r, int ctn);
 
 int ReaderCTAPI_GetCtn(const LCD_READER *r);
 void ReaderCTAPI_SetCtn(LCD_READER *r, int ctn);
