@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: readerctapi_p.h 23 2005-01-24 23:54:15Z aquamaniac $
+    cvs         : $Id: readerctapi_p.h 361 2007-04-28 11:46:38Z christian $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -23,7 +23,7 @@ struct READER_CTAPI {
   int ctn;
 };
 
-void ReaderCTAPI_freeData(void *bp, void *p);
+void GWENHYWFAR_CB ReaderCTAPI_freeData(void *bp, void *p);
 
 
 #endif /* CHIPCARD_READER_CTAPI_P_H */
