@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: driverifd.h 122 2005-10-22 00:42:09Z aquamaniac $
+    cvs         : $Id: driverifd.h 378 2007-05-02 08:41:05Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -18,7 +18,7 @@ typedef struct DRIVER_IFD DRIVER_IFD;
 
 
 #include <gwenhywfar/libloader.h>
-#include "driver_l.h"
+#include <chipcard/server/driver/driver.h>
 
 LCD_DRIVER *DriverIFD_new(int argc, char **argv);
 int DriverIFD_Start(LCD_DRIVER *d);
