@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: global.h 23 2005-01-24 23:54:15Z aquamaniac $
+    cvs         : $Id: global.h 378 2007-05-02 08:41:05Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -15,9 +15,8 @@
 #define RSACARD_GLOBAL_H
 
 
-#include <chipcard2/chipcard2.h>
-#include <chipcard2-client/client/client.h>
-#include <chipcard2-client/cards/geldkarte.h>
+#include <chipcard/chipcard.h>
+#include <chipcard/client/client.h>
 
 #include <gwenhywfar/logger.h>
 #include <gwenhywfar/db.h>
@@ -39,6 +38,7 @@ void showError(LC_CARD *card, LC_CLIENT_RESULT res, const char *x);
 int listReaders(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs);
 int checkReaders(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs);
 int getAtr(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs);
+int monitor(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs);
 
 
 
