@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: global.h 23 2005-01-24 23:54:15Z aquamaniac $
+    cvs         : $Id: global.h 453 2007-11-19 01:25:10Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -15,11 +15,12 @@
 #define RSACARD_GLOBAL_H
 
 
-#include <chipcard2/chipcard2.h>
-#include <chipcard2-client/client/client.h>
-#include <chipcard2-client/cards/kvkcard.h>
+#include <chipcard/chipcard.h>
+#include <chipcard/client/client.h>
+#include <chipcard/client/cards/kvkcard.h>
 
 #include <gwenhywfar/logger.h>
+#include <gwenhywfar/process.h>
 
 
 #define RETURNVALUE_PARAM   1
@@ -28,8 +29,16 @@
 #define RETURNVALUE_DEINIT  4
 
 
+void usage(const char *name, const char *ustr);
 void showError(LC_CARD *card, LC_CLIENT_RESULT res, const char *x);
 
+void okBeep();
+void errorBeep();
+
+
+int kvkRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs);
+int kvkDaemon(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs);
+
 
 
 
