@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: examplecard.h 34 2005-01-26 17:19:10Z aquamaniac $
+    cvs         : $Id: examplecard.h 378 2007-05-02 08:41:05Z martin $
     begin       : Sun Jun 13 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -21,7 +21,7 @@
 #ifndef CHIPCARD_CARD_EXAMPLECARD_H
 #define CHIPCARD_CARD_EXAMPLECARD_H
 
-#include <chipcard2-client/client/card.h>
+#include <chipcard/client/card.h>
 
 
 int ExampleCard_ExtendCard(LC_CARD *card);
